(* ------------------------------------------------------ *)
(*                     CRDISK.MOD                         *)
(* Verschlsselt Disketten, die nicht kopiergeschtzt sind*)
(*        (c) 1989  Ralf Hensmann  &  TOOLBOX             *)
(* ------------------------------------------------------ *)
MODULE CrDisk;

IMPORT Lib, IO, Str;

FROM SYSTEM   IMPORT Registers;
FROM DiskRead IMPORT ReadDisk, WriteDisk;
FROM FEAL     IMPORT KeyStr, SetKey, Crypt, DeCrypt;

CONST
  BufMax      = 8000;
                    (* Anzahl der 8 Byte-Blcke im Puffer *)

TYPE
  Bit64       = SET OF [0..63];
                          (* [1..64] --> SIZE = 9 Bytes ! *)
  TrackBuffer = ARRAY [1..BufMax] OF Bit64;

TYPE
  BootSektor = RECORD
                 Jmp         : ARRAY [0..2] OF BYTE;
                 Version     : ARRAY [1..8] OF CHAR;
                 ByPerSec    : CARDINAL;
                 SecPerClus  : SHORTCARD;
                 CountRes    : CARDINAL;
                 CountFAT    : BYTE;
                 CountMEntry : CARDINAL;
                 CountSecVol : CARDINAL;
                 MediaDesc   : SHORTCARD;
                 CountSecFAT : CARDINAL;
                 SecTrack    : CARDINAL;
                 CountHead   : CARDINAL;
                 HiddenSec   : CARDINAL;
                 Booter      : ARRAY [1..482] OF BYTE;
               END;
VAR
  Buffer     : TrackBuffer;
  Pass       : ARRAY [0..23] OF CHAR;         (* Passwort *)
  DoCrypt, Lw: CHAR;
  Error      : SHORTCARD;
  Hilf, Lauf,
  Block, Done,
  BufSector,
  BlockPerSec: CARDINAL;
  UeberT     : Bit64;
  B          : BootSektor;

  PROCEDURE Beep;
  BEGIN
    Lib.Sound(800);  Lib.Delay(100);
    Lib.NoSound;     Lib.Delay(30);
  END Beep;

  PROCEDURE Wait();
  VAR
    Ch : CHAR;
  BEGIN
    WHILE IO.KeyPressed() DO
      Ch := IO.RdKey();
    END;
    Ch := IO.RdKey();
    IO.WrLn;
  END Wait;

  PROCEDURE GetKey(P1, P2, P3 : CHAR) : CHAR;
  VAR
    Ch : CHAR;
  BEGIN
    REPEAT
      Ch := CAP(IO.RdKey());
    UNTIL (Ch = P1) OR (Ch = P2) OR (Ch = P3);
    IO.WrChar(Ch);
    IO.WrLn;
    RETURN Ch;
  END GetKey;

  PROCEDURE InitDrive(Lw : CHAR);
  VAR
    R : Registers;
  BEGIN
    R.AH := 32H;
    R.DL := SHORTCARD(CAP(Lw)) - 64;
    Lib.Dos(R);
    IF R.AL = 0FFH THEN
      Lib.FatalError("Laufwerk existiert nicht");
    END;
  END InitDrive;

  PROCEDURE CryptTrack( Lw          : CHAR;
                        StartSector : CARDINAL;
                        NumSector   : CARDINAL;
                        DoCrypt     : CHAR;
                        VAR UeberT  : Bit64) : SHORTCARD;
  VAR
    Error     : SHORTCARD;
    Lauf      : CARDINAL;
    HilfUeber : Bit64;
  BEGIN
    Error := ReadDisk(Lw, StartSector,
                      NumSector, ADR( Buffer ));
    IF Error # 0 THEN
      RETURN Error;
    END;
    IF DoCrypt = "V" THEN
      Buffer[1] := Buffer[1] / UeberT;
      Crypt(ADR(Buffer[1]));
      FOR Lauf := 2 TO NumSector * BlockPerSec DO
        Buffer[Lauf] := Buffer[Lauf] / Buffer[Lauf-1];
        Crypt(ADR(Buffer[Lauf]));
      END;
      UeberT := Buffer[NumSector * BlockPerSec];
    ELSE
      FOR Lauf := 1 TO NumSector * BlockPerSec DO
        HilfUeber := Buffer[Lauf];
        DeCrypt(ADR(Buffer[Lauf]));
        Buffer[Lauf] := Buffer[Lauf] / UeberT;
        UeberT := HilfUeber;
      END;
    END;
    Error := WriteDisk(Lw, StartSector,
                       NumSector, ADR(Buffer));
    RETURN Error;
  END CryptTrack;

BEGIN
  IO.WrStr("FEAL 2.0 Verschlsselungsprogramm fr ")
  IO.WrStr("Disketten");  IO.WrLn;
  IO.WrStr(" 1989 Ralf Hensmann & TOOLBOX ");
  IO.WrLn;  IO.WrLn;

  IO.WrStr(" V(erschlsseln, E(ntschlsseln, Q(uit? ");
  DoCrypt := GetKey("V", "E", "Q" );
  IF DoCrypt = "Q" THEN
    HALT;
  END;
  IO.WrLn;

  IO.WrStr(" Passwort : ");  IO.RdStr(Pass);
  Hilf := Str.Length(Pass) + 1;
  FOR Lauf := Hilf TO 23 DO
    Pass[Lauf] := Pass[Lauf-Hilf];  (* Passwort erweitern *)
  END;
                                (* Passwort verschlsseln *)
  SetKey(KeyStr(Pass[16]));  Crypt(ADR(Pass[ 8]));
  SetKey(KeyStr(Pass[ 8]));  Crypt(ADR(Pass[ 0]));
  SetKey(KeyStr(Pass[ 0]));
  IO.WrLn;

  IO.WrStr(" Laufwerk ? ");
  REPEAT
    Lw := CAP(IO.RdKey());
  UNTIL (Lw >= "A");

  IO.WrCharRep(10C, 12);
  IO.WrStr(" Bitte Diskette in Laufwerk "); IO.WrChar( Lw);
  IO.WrStr(" einlegen."); IO.WrLn;
  Wait;

  InitDrive(Lw);
  (* Bootsektor lesen *)
  Error := ReadDisk(Lw, 0, 1, ADR(B));
  IF (Error <> 0) OR (B.MediaDesc < 0F0H) THEN
    Lib.FatalError("Fehler beim Lesen des Boot-Sektors");
  END;
  BlockPerSec := B.ByPerSec DIV 8;
  IF ((B.ByPerSec MOD 8) # 0) OR (BlockPerSec > BufMax) THEN
    Lib.FatalError("Anzahl der Bytes pro Sektor kann " +
                   "nicht verarbeitet werden!" );
  END;

  BufSector := BufMax DIV BlockPerSec;

  UeberT := Bit64{};
  Done := B.CountRes;
  IO.WrStr(" Bereits ");
  IF DoCrypt = "V" THEN
    IO.WrStr("ver")
  ELSE
    IO.WrStr("ent");
  END;
  IO.WrStr("schlsselt:   0%");
  WHILE Done < B.CountSecVol DO
    Block := B.CountSecVol - Done;
    IF Block > BufSector THEN
      Block := BufSector;
    END;
    Error := CryptTrack(Lw, Done, Block, DoCrypt, UeberT);
    IF ( Error # 0 ) THEN
      IO.WrStr(" Sektor: ");    IO.WrCard(Done, 1);
      IO.WrStr("   Fehler : "); IO.WrShtCard(Error, 1);
      IO.WrLn;
      Beep;  Beep;  Beep;
      Lib.FatalError(" Diskettenfehler... " +
                     "Programm abgebrochen");
    END;
    INC(Done, Block);
    IO.WrCharRep(10C, 4);
    IO.WrLngCard((LONGCARD(Done)*100) DIV
                  LONGCARD(B.CountSecVol), 3);
    IO.WrStr("%");
  END;
  IO.WrLn;
  IF DoCrypt = "V" THEN
    IO.WrLn;
    IO.WrStr(" Merken Sie sich ihren Schlssel gut !!!");
    IO.WrLn;
  END;

END CrDisk.
(* ------------------------------------------------------ *)
(*                  Ende von CRDISK.MOD                   *)
